﻿[CmdletBinding()]
param(
    [Parameter(Mandatory = $true, HelpMessage = "EvOps Storage Connection String")]
    [string]$ConnectionString,
    [Parameter(Mandatory = $false, 
        HelpMessage = "Input the full filePath of the AzCopy.exe, e.g.: C:\Program Files (x86)\Microsoft SDKs\Azure\AzCopy\AzCopy.exe")]
    [String]$AzCopyPath = "C:\Program Files (x86)\Microsoft SDKs\Azure\AzCopy\AzCopy.exe",
    [Parameter(Mandatory = $false, 
        HelpMessage = "Input the path to where you want to export the storage data.")]
    [String]$Destination
)

$ErrorActionPreference = "Stop"

function ExportTableStorage {
    param (
        $AzCopyPath,
        $TableUri,
        $TableName,
        $Destination,
        $StorageKey
    )

    # AzCopy /Source:https://myaccount.table.core.windows.net/myTable/ /Dest:C:\myfolder\ /SourceKey:key
    $azCopyCmd = [string]::Format("""{0}"" /source:{1} /dest:{2} /sourcekey:""{3}"" /Manifest:""{4}"" /Y", $AzCopyPath, $TableUri, $Destination, $StorageKey, $TableName)
   
    Write-Verbose $azCopyCmd
    
    $result = cmd /c $azCopyCmd
    foreach ($s in $result) {
        Write-Host $s 
    }

    # If transfer failed, retry it
    while (($LASTEXITCODE -ne 0) -and ($RetryTimes -eq -1 -or $retry -le $retryTimes)) {
        Write-Host "Retry $retry : $azCopyCmd"
        $result = cmd /c $azCopyCmd
        foreach ($s in $result) {
            Write-Host $s 
        }
        $retry++
    }

    # If transfer failed after retry, print error
    if ($LASTEXITCODE -ne 0) {
        Write-Error "Table copy failed: $($container.Name)";
    }
}

function ExportBlobStorage {
    param (
        $AzCopyPath,
        $BlobUri,
        $Destination,
        $StorageKey
    )
    # AzCopy.exe /source:https://myaccount.blob.local.azurestack.external/mycontainer /dest:C:\myfolder /sourcekey:<key> /S
    $azCopyCmd = [string]::Format("""{0}"" /source:{1} /dest:{2} /sourcekey:""{3}"" /Y /S", $AzCopyPath, $BlobUri, $Destination, $StorageKey)
    
    Write-Verbose $azCopyCmd
    
    $result = cmd /c $azCopyCmd
    foreach ($s in $result) {
        Write-Host $s 
    }

    # If transfer failed, retry it
    while (($LASTEXITCODE -ne 0) -and ($RetryTimes -eq -1 -or $retry -le $retryTimes)) {
        Write-Host "Retry $retry : $azCopyCmd"
        $result = cmd /c $azCopyCmd
        foreach ($s in $result) {
            Write-Host $s 
        }
        $retry++
    }

    # If tranfer failed after retry, print error
    if ($LASTEXITCODE -ne 0) {
        Write-Error "Table copy failed: $($container.Name)";
    }
}

if (!$PSBoundParameters.ContainsKey('Destination') -or !(Test-Path $Destination)) {
    Write-Warning "Destination path not found, using current directory"
    $Destination = Resolve-Path ".\"
}
$ExportDirectory = New-Item -Path (Join-Path -Path $Destination -ChildPath ([DateTime]::UtcNow.Ticks)) -ItemType Directory

if (Test-Path $AzCopyPath) {
    $FileItemVersion = (Get-Item $AzCopyPath).VersionInfo
    $FilePath = ("{0}.{1}.{2}.{3}" -f $FileItemVersion.FileMajorPart, $FileItemVersion.FileMinorPart, $FileItemVersion.FileBuildPart, $FileItemVersion.FilePrivatePart)

    if ([version] $FilePath -lt "7.0.0.2") {
        $AzCopyPath = Read-Host "Version of AzCopy found at default install directory is of a lower, unsupported version. Please input the full filePath of the AzCopy.exe that is between version 7.0.0.2 and 7.9.9.9, e.g.: C:\Program Files (x86)\Microsoft SDKs\Azure\AzCopy\AzCopy.exe"
    }

    if ([version] $FilePath -gt "7.9.9.9") {
        $AzCopyPath = Read-Host "Version of AzCopy found at default install directory is of a higher, unsupported version. Please input the full filePath of the AzCopy.exe that is between version 7.0.0.2 and 7.9.9.9, e.g.: C:\Program Files (x86)\Microsoft SDKs\Azure\AzCopy\AzCopy.exe"
    }
}
else {
    $AzCopyPath = Read-Host "Input the full filePath of the AzCopy.exe, e.g.: C:\Program Files (x86)\Microsoft SDKs\Azure\AzCopy\AzCopy.exe"
}

$StorageContext = New-AzureStorageContext -ConnectionString $ConnectionString

# Enumarate tables
$AccountKey = $ConnectionString.Split(";")|Where-Object {$_ -like "AccountKey*"}
$Key = $AccountKey.Replace("AccountKey=", "")

$Tables = Get-AzureStorageTable -Context $StorageContext

$excludedTables = @(
    "ExtensionHealth",
    "ExtensionLogs"    
)

$i = 1
foreach ($Table in $Tables) {
    if ($excludedTables.Contains($Table.Name)) {
        continue
    }
    Write-Progress -Activity "Exporting table" -PercentComplete ($i / $Tables.Count * 100)
    ExportTableStorage -AzCopyPath $AzCopyPath -TableUri $Table.Uri.AbsoluteUri -Destination (Join-Path -Path $ExportDirectory.FullName -ChildPath "Tables") -StorageKey $Key -TableName $Table.Name
    $i++;
}

$BlobContainers = Get-AzureStorageContainer -Context $StorageContext

$i = 1
foreach ($BlobContainer in $BlobContainers) {
    Write-Progress -Activity "Exporting blob" -PercentComplete ($i / $Tables.Count * 100)
    ExportBlobStorage -AzCopyPath $AzCopyPath -BlobUri "$($StorageContext.BlobEndPoint)$($BlobContainer.Name)" -Destination (Join-Path -Path $ExportDirectory.FullName -ChildPath "Blobs" | Join-Path -ChildPath $BlobContainer.Name ) -StorageKey $Key
    $i++;
}

# Compress archive
Compress-Archive -Path "$($ExportDirectory.FullName)\*" -DestinationPath "$Destination\$($ExportDirectory.Name).zip"

# Remove uncompressed files
Remove-Item -Path $ExportDirectory.FullName -Recurse

