[CmdletBinding()]
param(
    [Parameter(Mandatory = $true, HelpMessage = "EvOps Storage Connection String")]
    [string]$ConnectionString,
    [Parameter(Mandatory = $false, HelpMessage = "Input the full filePath of the AzCopy.exe, e.g.: C:\Program Files (x86)\Microsoft SDKs\Azure\AzCopy\AzCopy.exe")]
    [String]$AzCopyPath = "C:\Program Files (x86)\Microsoft SDKs\Azure\AzCopy\AzCopy.exe",
    [Parameter(Mandatory = $true, HelpMessage = "Input the path to the backup zip")]
    [String]$BackupPath
)

function Restore-Tables {
    param(
        $TableFolder, 
        $TableEndpoint, 
        $DestKey
    ) 
    Write-Verbose "Restoring tables"
    $manifestFiles = Get-ChildItem $TableFolder  *.manifest
    $i = 1
    foreach ($manifest in $manifestFiles) {
        Write-Progress -Activity "Restoring table" -PercentComplete ($i / $manifestFiles.Count * 100)
        $tableName = [System.IO.Path]::GetFileNameWithoutExtension($manifest)
        $tableUri = "$TableEndpoint$tableName"
        $azCopyCmd = [string]::Format("""{0}"" /Source:{1} /Dest:{2} /DestKey:""{3}"" /Manifest:""{4}"" /EntityOperation:InsertOrReplace /Y", $AzCopyPath, $TableFolder, $tableUri, $DestKey, $manifest.Name)
        Write-Verbose $azCopyCmd
    
        cmd /c $azCopyCmd
        $i++
    }
}

function Restore-Blobs {
    param(
        $BlobFolder, 
        $BlobEndpoint, 
        $DestKey
    ) 
    Write-Verbose "Restoring blobs folder: $BlobFolder"

    $containerFolders = Get-ChildItem $BlobFolder -Directory
    
    $i = 1
    foreach ($containerFolder in $containerFolders) {

        Write-Progress -Activity "Restoring container $containerFolder" -PercentComplete ($i / $containerFolders.Count * 100)
        $containerName = $containerFolder.Name
        $blobUri = "$BlobEndpoint$containerName"
        $azCopyCmd = [string]::Format("""{0}"" /Source:{1} /Dest:{2} /DestKey:""{3}"" /S /Y", $AzCopyPath, $containerFolder.FullName, $blobUri, $DestKey)
        Write-Verbose $azCopyCmd
    
        cmd /c $azCopyCmd
        $i++
    }
}

function New-TemporaryDirectory {
    $parent = [System.IO.Path]::GetTempPath()
    [string] $name = [System.Guid]::NewGuid()
    New-Item -ItemType Directory -Path (Join-Path $parent $name)
}

try {
    $tempPath = New-TemporaryDirectory
    $zipFileName = (Get-Item $BackupPath).Name
    Write-Verbose "Created temp folder: $tempPath"
    Copy-Item $BackupPath -Destination $tempPath
    $zipTempPath = Join-Path $tempPath -ChildPath $zipFileName
    Expand-Archive $zipTempPath -DestinationPath $tempPath
    $TableFolder = Join-Path $tempPath -ChildPath "Tables"

    $storageContext = New-AzureStorageContext -ConnectionString $ConnectionString
    $accountKey = $ConnectionString.Split(";")|Where-Object {$_ -like "AccountKey*"}
    $key = $accountKey.Replace("AccountKey=", "")
    Restore-Tables -tableFolder $TableFolder -TableEndpoint $storageContext.TableEndPoint -DestKey $key

    $BlobFolder = Join-Path $tempPath -ChildPath "Blobs"

    Restore-Blobs -blobFolder $BlobFolder -BlobEndpoint $storageContext.BlobEndPoint -DestKey $key
}
catch {
}
finally {
    #Delete temp folder
    Remove-Item $tempPath -Recurse    
}
